/*
 * Copyright (c) 2019-2020. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.scanner;

import java.util.HashMap;
import java.util.Map;

import org.hedgecode.chess.scanner.portal.ChessGamesScanner;

/**
 * ChessHogScanner
 *
 * @author Dmitry Samoshin aka gotty
 */
public final class ChessHogScanner {

    private static ChessHogScanner _instance;

    private final Map<ScannerType, Scanner> SCANNERS = new HashMap<ScannerType, Scanner>() {
        {
            put(ScannerType.CHESSGAMES, new ChessGamesScanner());
        }
    };

    public static Scanner get(ScannerType type) {
        return getInstance().scanner(type);
    }

    private void init() {
        try {
            for (Scanner scanner : SCANNERS.values()) {
                if (scanner instanceof Initiable) {
                    Initiable initiableScanner = (Initiable) scanner;
                    initiableScanner.init();
                }
            }
        } catch (ChessHogScannerException e) {
            throw new RuntimeException(e);
        }
    }

    private Scanner scanner(ScannerType type) {
        return SCANNERS.get(type);
    }

    private ChessHogScanner() {
    }

    private static ChessHogScanner getInstance() {
        if (_instance == null) {
            _instance = new ChessHogScanner();
            _instance.init();
        }
        return _instance;
    }

}
